==> rook-0.2.6.tar.gz: rook-v0.2.6/main.go <==
package main

//go:generate go tool ser1.net/claphelp/v2/cmd/makeclapman --d man1 --author "Sean E. Russell <ser@ser1.net>" --description "Rook allows you to use a KeePass v2 database as storage for secrets. It provides client and server modes; the server unlocks the database and stays in memory, while the client communicates over a socket with the server and fetches data." --run "./ --help"

import (
	"bufio"
	"bytes"
	"encoding/base64"
	"fmt"
	"io"
	"log"
	"net"
	"os"
	"os/exec"
	"path"
	"strings"

	"ser1.net/claptrap/v4"
)

const (
	LISTCMDS                = "commands"
	SERVE                   = "serve"
	OPEN                    = "open"
	LOCK                    = "lock"
	LIST                    = "list"
	SHOW                    = "show"
	SEARCH                  = "search"
	MATCH                   = "match"
	EXIT                    = "exit"
	STATUS                  = "status"
	INFO                    = "info"
	RELOAD                  = "reload"
	SEARCHTERM1             = "titleOrPath"
	SEARCHTERM2             = "title"
	PATH                    = "path"
	TITLE                   = "title"
	ENV_PASSWORD            = "ROOK_PASSWORD"
	DBPATH                  = "databasePath"
	PASSWORD                = "password"
	TREE                    = "tree"
	OTPIN                   = "otpin"
	TRUE                    = "true"
	SUCCESS                 = "SUCCESS"
	MAX_PIN_UNLOCK_ATTEMPTS = 3
	KEYDATA                 = "keydata"
)

var Version string = "0.0.3"
var COMMANDS map[string]bool
var socketPath func() string

// TODO cgroups to further constrain access to the socket. @maybe
// TODO Allow the user to provide a program to run to request a password to unlock the DB if it is locked. @maybe
// TODO Add kpmenu mode, where server is pre-configured with commands it can run, and the client just asks it to do things.
// TODO Add YubiKey credential support
// TODO (A) Add -1 to get closest (Levenshtein) match
// TODO (A) https://man7.org/linux/man-pages/man7/keyrings.7.html, or namespaces
func main() {
	COMMANDS = map[string]bool{
		LISTCMDS: true,
		OPEN:     true,
		LOCK:     true,
		LIST:     true,
		SHOW:     true,
		SEARCH:   true,
		MATCH:    true,
		EXIT:     true,
		INFO:     true,
		STATUS:   true,
		RELOAD:   true,
	}
	root := setupFlags()
	root.Parse(nil)

	sp := path.Join(os.Getenv("HOME"), ".rook.sock")
	if root.String("socket") != "" {
		sp = root.String("socket")
	}
	socketPath = func() string {
		return sp
	}
	log.SetOutput(os.Stderr)

	var keydata []byte
	var command string
	if root.Command != nil {
		command = root.Command.Name
	}
	if root.Command != nil {
		if root.Command.Bool("prompt") {
			passwd := getPassword("Password: ")
			root.Args["password"].Set(passwd)
		} else if root.Command.Bool("stdin") {
			passwd, err := io.ReadAll(os.Stdin)
			if err != nil {
				log.Fatalf("instructed to read password from stdin, but unable to: %s", err)
			}
			root.Args["password"].Set(strings.TrimSpace(string(passwd)))
		}
		if f := root.Command.String("keyfile"); f != "" {
			var err error
			keydata, err = os.ReadFile(f)
			if err != nil {
				log.Fatalf("error reading keyfile: %s", err)
			}
		}
	}

	if root.Bool("help") || command == "help" {
		if root.Command != nil {
			if root.Command.Name != "help" {
				claptrap.RootCommand = root.Command
			} else {
				command := root.Command.String("command")
				if command != "" {
					claptrap.RootCommand = root.Commands[command]
				}
			}
		}
		claptrap.Usage()
		os.Exit(0)
	} else if command == "version" || root.Bool("version") {
		fmt.Printf("rook ")
		if !strings.HasPrefix(Version, "v") && !strings.HasPrefix(Version, "V") {
			fmt.Printf("v")
		}
		fmt.Printf("%s\n", Version)
	} else if command == SERVE {
		server := new(root, keydata)
		if root.Command.Bool("secure") {
			server.pin = "XXXX"
		}
		// Make sure we're not holding onto the password
		root.Args["password"].Set("")
		if root.Command.Bool("detach") {
			for i, a := range os.Args {
				if a == "-d" || a == "--detach" {
					os.Args[i] = "--no-detach"
				}
			}
			cmd := exec.Command(os.Args[0], os.Args[1:]...)
			cmd.Stdout = os.Stdout
			cmd.Stdin = os.Stdin
			cmd.Start()
			os.Exit(0)
		} else {
			server.start()
		}
	} else {
		if !COMMANDS[command] {
			log.Fatalf("unknown command %s", command)
		}
		args := []string{command}
		if root.Bool("pin") {
			var pins string
			fmt.Scanln(&pins)
			args = append(args, pins)
		}
		switch command {
		default:
			if !processCommand(false, args...) {
				os.Exit(1)
			}
		case LIST:
			if tree := root.Command.Bool("tree"); tree {
				args = append(args, TREE, TRUE)
			}
			if !processCommand(false, args...) {
				os.Exit(1)
			}
		case LOCK:
			if root.Command.Bool("pin") {
				args = append(args, OTPIN, TRUE)
			}
			if !processCommand(false, args...) {
				os.Exit(1)
			}
		case OPEN:
			if dbpath := root.Command.String("path"); dbpath != "" {
				args = append(args, DBPATH, dbpath)
			}
			if root.Command.Args["pin"].IsSet() {
				pin := root.Command.String("pin")
				args = append(args, OTPIN, pin)
			}
			password := root.String("password")
			if password == "" {
				password = os.Getenv(ENV_PASSWORD)
			}
			args = append(args, PASSWORD, password)
			if len(keydata) > 0 {
				e := base64.StdEncoding.EncodeToString(keydata)
				args = append(args, KEYDATA, e)
			}
			if !processCommand(false, args...) {
				os.Exit(1)
			}
		case SEARCH:
			if root.Command.Args[SEARCHTERM2].IsSet() {
				args = append(args, TITLE, root.Command.String(SEARCHTERM2), PATH, root.Command.String(SEARCHTERM1))
			} else {
				args = append(args, TITLE, root.Command.String(SEARCHTERM1))
			}
			if !processCommand(false, args...) {
				os.Exit(1)
			}
		case MATCH:
			args = append(args, SEARCHTERM1, root.Command.String("title"))
			if !processCommand(false, args...) {
				os.Exit(1)
			}
		case SHOW:
			com := root.Command
			var path string
			title := root.Command.String(SEARCHTERM1)
			if root.Command.Args[SEARCHTERM2].IsSet() {
				path = title
				title = root.Command.String(SEARCHTERM2)
			}
			if path != "" {
				args = append(args, PATH, path)
			}
			if title != "" {
				args = append(args, TITLE, title)
			} else if com.Args["i"].IsSet() {
				args = append(args, "index", fmt.Sprintf("%d", com.Int("i")))
			}
			if title == "" && !com.Args["i"].IsSet() {
				fmt.Println("either a search term or an index is required")
				claptrap.Usage()
				os.Exit(1)
			}
			if com.Bool("password") {
				args = append(args, "field", "password")
			} else if com.Bool("user") {
				args = append(args, "field", "username")
			} else if com.Bool("otp") {
				args = append(args, "field", "otp")
			} else if com.Args["field"].IsSet() {
				args = append(args, "field", com.String("field"))
			}
			if !processCommand(com.Bool("no-eol"), args...) {
				os.Exit(1)
			}
		}
	}
}

func processCommand(trim bool, args ...string) (gotResults bool) {
	var rw net.Conn
	var err error
	if rw, err = net.Dial("unix", socketPath()); err != nil {
		log.Fatalf("[FATAL] unable to connect to the server socket: %s", err)
	}
	pin := os.Getenv("ROOK_PIN")
	if pin != "" {
		args = append([]string{pin}, args...)
	}
	output := strings.Join(args, "\r")
	rw.Write([]byte(fmt.Sprintf("%s\000", output)))

	rdr := bufio.NewReader(rw)
	line, err := rdr.ReadBytes('\000')
	line = bytes.TrimRight(bytes.TrimSpace(line), "\000")
	if err != nil {
		log.Printf("error encountered parsing string: %s", err)
	}
	if strings.HasPrefix(string(line), "FAIL\f") {
		fmt.Println(strings.ReplaceAll(string(line), "\f", "\n"))
		return false
	}
	sline := strings.TrimRight(string(line), "\r\f")
	if len(sline) == 0 {
		return false
	}
	records := strings.Split(sline, "\f")
	if len(records) > 0 {
		gotResults = true
	}
	for _, record := range records {
		sline := strings.Trim(record, "\r")
		parts := strings.Split(sline, "\r")
		if len(parts) < 1 {
			log.Printf("empty message")
		}
		if len(parts) == 1 {
			if trim {
				fmt.Print(parts[0])
			} else {
				fmt.Println(parts[0])
			}
		} else {
			for _, txt := range parts {
				fmt.Printf("%s\n", txt)
			}
		}
		if record != records[len(records)-1] {
			fmt.Println()
		}
	}
	return gotResults
}

func setupFlags() *claptrap.CommandConfig {
	root := claptrap.Command("rook", "A secret service tool backed by a Keepass v4 database.")
	root.Add("--help", "-h", "Prints usage text")
	root.AddCommand("help", "Prints usage text").Add("command", "get more info about this command")
	root.Add("--password", "", "Do not use this argument. It's a horrible idea.")
	root.Add("--force", "-f", "purge any existing server socket first. If there's already a server running, this will probably break it.")
	root.Add("--version", "-v", "print the version and exit")
	root.Add("--socket", "", "an alternate client/server socket from the default (allows running multiple rook servers). Must be a full file name, e.g. /tmp/rook2.sock")
	root.Add("--pin", "Read the secret pin from the STDIN")
	root.AddCommand("version", "print out the version and exit")
	serve := root.AddCommand(SERVE, "start the server")
	serve.Add("--prompt", "-P", "prompt for the database password")
	serve.Add("--keyfile", "-k", "", "read a keyfile from this supplied path")
	serve.Add("--detach", "-d", "fork the process into the background and detach")
	serve.Add("--secure", "run in secure mode, requiring a pin for every interaction. The client must be called with --pin, and the value is always read from STDIN")
	serve.Add("--stdin", "Read the database password from stdin.")
	serve.Add("database", "the path to a Keepass v4 database. If not provided, one must be opened with the 'open' command from the client")
	open := root.AddCommand(OPEN, "open a new database, or unlock the existing one. The password may either be prompted for, or may be supplied in a ROOK_PASSWORD environment variable.")
	open.Add("--prompt", "-P", "prompt for the database password")
	open.Add("--keyfile", "-k", "", "read a keyfile from this supplied path")
	open.Add("--pin", "", "a one-time pin provided by a previous 'lock' that will re-unlock the database")
	open.Add("path", "full path to the database (the server must be able to find the file)")
	root.AddCommand(LOCK, "lock an open database").Add("--pin", "create a one-time pin code that can unlock the DB; the pin will be returned")
	root.AddCommand(LIST, "list entries in the database").Add("--tree", "-t", "show database entries as a tree")
	root.AddCommand(LISTCMDS, "list all of the supported server commands")
	root.AddCommand(EXIT, "kill background process")
	root.AddCommand(INFO, "print database information")
	root.AddCommand(STATUS, "report whether the DB is unlocked. Lighter call than info")
	root.AddCommand(RELOAD, "reload the database. This shouldn't be necessary as rook detects file system mod time changes and auto-refreshes, but this command can force it.")
	search := root.AddCommand(SEARCH, "list entries with title, URL, or tag matching a string")
	search.Add("!titleOrPath", "string for matching titles (or paths). If the second argument is provided, this matches the path.")
	search.Add("title", "string for matching titles. If provided, the previous argument filters the path.")
	root.AddCommand("match", "find entries who have titles, urls, or autocomplete entries matching the supplied string. This is the opposite of search, in that the entry records must be substrings of the argument.").Add("title", "string to match against")
	show := root.AddCommand(SHOW, "show the contents of an entry")
	show.Add("titleOrPath", "string for matching titles (or paths). If the second argument is provided, this matches the path.")
	show.Add("title", "string for matching titles. If provided, the previous argument filters the path.")
	show.Add("-i", 0, "show the entry with this index")
	show.Add("--no-eol", "-n", "omit the end-of-line character")
	show.Add("--field", "-f", "", "print only the value of this field")
	show.Add("--password", "-p", "print only the password")
	show.Add("--otp", "-o", "generate a one-time-pass code from the entry")
	show.Add("--user", "-u", "print only the user name")
	return root
}

==> rook-0.3.0.tar.gz: rook-v0.3.0/main.go <==
package main

//go:generate go tool ser1.net/claphelp/v2/cmd/makeclapman --d man1 --author "Sean E. Russell <ser@ser1.net>" --description "Rook allows you to use a KeePass v2 database as storage for secrets. It provides client and server modes; the server unlocks the database and stays in memory, while the client communicates over a socket with the server and fetches data." --run "./ --help"

import (
	"bufio"
	"bytes"
	"encoding/base64"
	"fmt"
	"io"
	"log"
	"net"
	"os"
	"os/exec"
	"path"
	"runtime"
	"strings"

	"ser1.net/claptrap/v4"
)

const (
	_listcmds             = "commands"
	_serve                = "serve"
	_open                 = "open"
	_lock                 = "lock"
	_list                 = "list"
	_show                 = "show"
	_search               = "search"
	_match                = "match"
	_exit                 = "exit"
	_status               = "status"
	_info                 = "info"
	_reload               = "reload"
	_searchterm1          = "titleOrPath"
	_searchterm2          = "title"
	_path                 = "path"
	_title                = "title"
	_envpassword          = "ROOK_PASSWORD"
	_dbpath               = "databasePath"
	_password             = "password"
	_tree                 = "tree"
	_otpin                = "otpin"
	_true                 = "true"
	_success              = "SUCCESS"
	_maxpinunlockattempts = 3
	_keydata              = "keydata"
)

// Version is the program version, exported so it can be set at compile time
var Version = "0.0.3"
var _commands map[string]bool
var socketPath func() string

// TODO Allow the user to provide a program to run to request a password to unlock the DB if it is locked. @maybe
// TODO Add kpmenu mode, where server is pre-configured with commands it can run, and the client just asks it to do things.
// TODO Add YubiKey credential support
// TODO (A) Add -1 to get closest (Levenshtein) match
// TODO (A) Add some unit tests. This is embarassing.
func main() {
	_commands = map[string]bool{
		_listcmds: true,
		_open:     true,
		_lock:     true,
		_list:     true,
		_show:     true,
		_search:   true,
		_match:    true,
		_exit:     true,
		_info:     true,
		_status:   true,
		_reload:   true,
	}
	root := setupFlags()
	root.Parse(nil)

	sp := path.Join(os.Getenv("HOME"), ".rook.sock")
	if root.String("socket") != "" {
		sp = root.String("socket")
	}
	socketPath = func() string {
		return sp
	}
	log.SetOutput(os.Stderr)

	var keydata []byte
	var command string
	if root.Command != nil {
		command = root.Command.Name
	}
	if root.Command != nil {
		if root.Command.Bool("prompt") {
			passwd := getPassword("Password: ")
			root.Args["password"].Set(passwd)
		} else if root.Command.Bool("stdin") {
			passwd, err := io.ReadAll(os.Stdin)
			if err != nil {
				log.Fatalf("instructed to read password from stdin, but unable to: %s", err)
			}
			root.Args["password"].Set(strings.TrimSpace(string(passwd)))
		}
		if f := root.Command.String("keyfile"); f != "" {
			var err error
			keydata, err = os.ReadFile(f)
			if err != nil {
				log.Fatalf("error reading keyfile: %s", err)
			}
		}
	}

	if root.Bool("help") || command == "help" {
		if root.Command != nil {
			if root.Command.Name != "help" {
				claptrap.RootCommand = root.Command
			} else {
				command := root.Command.String("command")
				if command != "" {
					claptrap.RootCommand = root.Commands[command]
				}
			}
		}
		claptrap.Usage()
		os.Exit(0)
	} else if command == "version" || root.Bool("version") {
		fmt.Printf("rook ")
		if !strings.HasPrefix(Version, "v") && !strings.HasPrefix(Version, "V") {
			fmt.Printf("v")
		}
		fmt.Printf("%s\n", Version)
	} else if command == _serve {
		server := newServer(root, keydata)
		if root.Bool("keyring") {
			server.keyring = true
			root.Command.Args["secure"].Set("true")
		}
		if root.Command.Bool("secure") {
			server.pin = "XXXX"
		}
		// Make sure we're not holding onto the password
		root.Args["password"].Set("")
		if root.Command.Bool("detach") {
			for i, a := range os.Args {
				if a == "-d" || a == "--detach" {
					os.Args[i] = "--no-detach"
				}
			}
			cmd := exec.Command(os.Args[0], os.Args[1:]...)
			cmd.Stdout = os.Stdout
			cmd.Stdin = os.Stdin
			cmd.Start()
			os.Exit(0)
		}
		server.start()
	} else {
		if !_commands[command] {
			log.Fatalf("unknown command %s", command)
		}
		args := []string{command}
		if root.Bool("pin") {
			var pins string
			fmt.Scanln(&pins)
			args = append(args, pins)
		}
		if root.Bool("keyring") {
			pin, err := getPin()
			if err != nil {
				log.Fatalf("error fetching pin from keyring: %v", err)
			}
			// Sometimes KEYCTL_READ appends a CR?
			pins := strings.TrimSpace(string(pin))
			args = append(args, pins)
		}
		switch command {
		default:
			if !processCommand(false, args...) {
				os.Exit(1)
			}
		case _list:
			if tree := root.Command.Bool("tree"); tree {
				args = append(args, _tree, _true)
			}
			if !processCommand(false, args...) {
				os.Exit(1)
			}
		case _lock:
			if root.Command.Bool("pin") {
				args = append(args, _otpin, _true)
			}
			if !processCommand(false, args...) {
				os.Exit(1)
			}
		case _open:
			if dbpath := root.Command.String("path"); dbpath != "" {
				args = append(args, _dbpath, dbpath)
			}
			if root.Command.Args["pin"].IsSet() {
				pin := root.Command.String("pin")
				args = append(args, _otpin, pin)
			}
			password := root.String("password")
			if password == "" {
				password = os.Getenv(_envpassword)
			}
			args = append(args, _password, password)
			if len(keydata) > 0 {
				e := base64.StdEncoding.EncodeToString(keydata)
				args = append(args, _keydata, e)
			}
			if !processCommand(false, args...) {
				os.Exit(1)
			}
		case _search:
			if root.Command.Args[_searchterm2].IsSet() {
				args = append(args, _title, root.Command.String(_searchterm2), _path, root.Command.String(_searchterm1))
			} else {
				args = append(args, _title, root.Command.String(_searchterm1))
			}
			if !processCommand(false, args...) {
				os.Exit(1)
			}
		case _match:
			args = append(args, _searchterm1, root.Command.String("title"))
			if !processCommand(false, args...) {
				os.Exit(1)
			}
		case _show:
			com := root.Command
			var path string
			title := root.Command.String(_searchterm1)
			if root.Command.Args[_searchterm2].IsSet() {
				path = title
				title = root.Command.String(_searchterm2)
			}
			if path != "" {
				args = append(args, _path, path)
			}
			if title != "" {
				args = append(args, _title, title)
			} else if com.Args["i"].IsSet() {
				args = append(args, "index", fmt.Sprintf("%d", com.Int("i")))
			}
			if title == "" && !com.Args["i"].IsSet() {
				fmt.Println("either a search term or an index is required")
				claptrap.Usage()
				os.Exit(1)
			}
			if com.Bool("password") {
				args = append(args, "field", "password")
			} else if com.Bool("user") {
				args = append(args, "field", "username")
			} else if com.Bool("otp") {
				args = append(args, "field", "otp")
			} else if com.Args["field"].IsSet() {
				args = append(args, "field", com.String("field"))
			}
			if !processCommand(com.Bool("no-eol"), args...) {
				os.Exit(1)
			}
		}
	}
}

func processCommand(trim bool, args ...string) (gotResults bool) {
	var rw net.Conn
	var err error
	if rw, err = net.Dial("unix", socketPath()); err != nil {
		log.Fatalf("[FATAL] unable to connect to the server socket: %s", err)
	}
	pin := os.Getenv("ROOK_PIN")
	if pin != "" {
		args = append([]string{pin}, args...)
	}
	output := strings.Join(args, "\r")
	rw.Write(fmt.Appendf(nil, "%s\000", output))

	rdr := bufio.NewReader(rw)
	line, err := rdr.ReadBytes('\000')
	line = bytes.TrimRight(bytes.TrimSpace(line), "\000")
	if err != nil {
		log.Printf("error encountered parsing string: %s", err)
	}
	if strings.HasPrefix(string(line), "FAIL\f") {
		fmt.Println(strings.ReplaceAll(string(line), "\f", "\n"))
		return false
	}
	sline := strings.TrimRight(string(line), "\r\f")
	if len(sline) == 0 {
		return false
	}
	records := strings.Split(sline, "\f")
	if len(records) > 0 {
		gotResults = true
	}
	for _, record := range records {
		sline := strings.Trim(record, "\r")
		parts := strings.Split(sline, "\r")
		if len(parts) < 1 {
			log.Printf("empty message")
		}
		if len(parts) == 1 {
			if trim {
				fmt.Print(parts[0])
			} else {
				fmt.Println(parts[0])
			}
		} else {
			for _, txt := range parts {
				fmt.Printf("%s\n", txt)
			}
		}
		if record != records[len(records)-1] {
			fmt.Println()
		}
	}
	return gotResults
}

func setupFlags() *claptrap.CommandConfig {
	root := claptrap.Command("rook", "A secret service tool backed by a Keepass v4 database.")
	root.Add("--help", "-h", "Prints usage text")
	root.AddCommand("help", "Prints usage text").Add("command", "get more info about this command")
	root.Add("--password", "", "Do not use this argument. It's a horrible idea.")
	root.Add("--force", "-f", "purge any existing server socket first. If there's already a server running, this will probably break it.")
	root.Add("--version", "-v", "print the version and exit")
	root.Add("--socket", "", "an alternate client/server socket from the default (allows running multiple rook servers). Must be a full file name, e.g. /tmp/rook2.sock")
	root.Add("--pin", "Read the secret pin from the STDIN")
	if runtime.GOOS == "linux" {
		root.Add("--keyring", "use the Linux kernel keyring to store the pin in the user session context. Enables (and overrides) --secure.")
	}
	root.AddCommand("version", "print out the version and exit")
	serve := root.AddCommand(_serve, "start the server")
	serve.Add("--prompt", "-P", "prompt for the database password")
	serve.Add("--keyfile", "-k", "", "read a keyfile from this supplied path")
	serve.Add("--detach", "-d", "fork the process into the background and detach")
	serve.Add("--secure", "run in secure mode, requiring a pin for every interaction. The client must be called with --pin, and the value is always read from STDIN")
	serve.Add("--stdin", "Read the database password from stdin.")
	serve.Add("database", "the path to a Keepass v4 database. If not provided, one must be opened with the 'open' command from the client")
	open := root.AddCommand(_open, "open a new database, or unlock the existing one. The password may either be prompted for, or may be supplied in a ROOK_PASSWORD environment variable.")
	open.Add("--prompt", "-P", "prompt for the database password")
	open.Add("--keyfile", "-k", "", "read a keyfile from this supplied path")
	open.Add("--pin", "", "a one-time pin provided by a previous 'lock' that will re-unlock the database")
	open.Add("path", "full path to the database (the server must be able to find the file)")
	root.AddCommand(_lock, "lock an open database").Add("--pin", "create a one-time pin code that can unlock the DB; the pin will be returned")
	root.AddCommand(_list, "list entries in the database").Add("--tree", "-t", "show database entries as a tree")
	root.AddCommand(_listcmds, "list all of the supported server commands")
	root.AddCommand(_exit, "kill background process")
	root.AddCommand(_info, "print database information")
	root.AddCommand(_status, "report whether the DB is unlocked. Lighter call than info")
	root.AddCommand(_reload, "reload the database. This shouldn't be necessary as rook detects file system mod time changes and auto-refreshes, but this command can force it.")
	search := root.AddCommand(_search, "list entries with title, URL, or tag matching a string")
	search.Add("!titleOrPath", "string for matching titles (or paths). If the second argument is provided, this matches the path.")
	search.Add("title", "string for matching titles. If provided, the previous argument filters the path.")
	root.AddCommand("match", "find entries who have titles, urls, or autocomplete entries matching the supplied string. This is the opposite of search, in that the entry records must be substrings of the argument.").Add("title", "string to match against")
	show := root.AddCommand(_show, "show the contents of an entry")
	show.Add("titleOrPath", "string for matching titles (or paths). If the second argument is provided, this matches the path.")
	show.Add("title", "string for matching titles. If provided, the previous argument filters the path.")
	show.Add("-i", 0, "show the entry with this index")
	show.Add("--no-eol", "-n", "omit the end-of-line character")
	show.Add("--field", "-f", "", "print only the value of this field")
	show.Add("--password", "-p", "print only the password")
	show.Add("--otp", "-o", "generate a one-time-pass code from the entry")
	show.Add("--user", "-u", "print only the user name")
	return root
}
